#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <functional>
using namespace std;
#ifdef WIN32
#include <windows.h>
#endif

const string::value_type g_delimiters[] = " \n\t\r',.;?!=<>-+/*";
const char g_version[] = "0.1.3";

void translate(string& s);
void usage(const char *progname, int do_exit = 0);
void version();

int main(int argc, char **argv)
{
	// fill the vector of words from cmd line
	vector<string> strs;
	for (int i = 1; i < argc; i ++)
		strs.push_back(argv[i]);

	// analyse the command line to see if there're special args
	if (strs.empty())
		usage(argv[0], 1);

	if (find(strs.begin(), strs.end(), "--help") != strs.end())
		usage(argv[0]);
	else if (find(strs.begin(), strs.end(), "--version") != strs.end())
		version();

	// do the translation on each word
	for_each(strs.begin(), strs.end(), translate);

	// output the result
	for (vector<string>::iterator i = strs.begin(); i != strs.end(); i ++)
	{
#ifdef WIN32
		char *buffer = new char[i->length() + 1];
		CharToOem(i->c_str(), buffer);
		cout << buffer << " ";
		delete[] buffer;
#else
		cout << *i << " ";
#endif
	}
	cout << endl;

	return 0;
}

void translate(string& s)
{
	string n;
	string::size_type pos;

	// try to find special characters which make a word in fact
	// two words for translation
	if ((pos = s.find_first_of(g_delimiters)) != string::npos)
	{
		// split in two words, a make an recursive call
		string s1, s2;
		string::value_type c;
		c = s.at(pos);			// the delimiter
		s1 = s.substr(0, pos);	// before delimiter
		s2 = s.substr(pos + 1);	// after delimiter
		translate(s1);			// recursive call for left part
		translate(s2);			//      ""        ""  right  ""
		n = s1;					// put the translated left part
		n.push_back(c);			// put the delimiter
		n.append(s2);			// put the translated right part
		s = n;					// replace the original
	}
	else
	{
		// do the translation
		reverse(s.begin(), s.end());
	}
}

void usage(const char *progname, int do_exit)
{
	cout << endl;
	cout << "usage: " << progname << " [args] <list of words to translate>" << endl;
	cout << "  with args which are optionnals and can be one of the following:" << endl;
	cout << "  --help    show this help screen and exit" << endl;
	cout << "  --version show the program version and exit" << endl;
	cout << endl;

	if (do_exit != -1)
		exit(do_exit);
}

void version()
{
	cout << "bulgroz version " << g_version << endl;
	cout << "Copyright (c) 2003 Richard GILL <richard@nasgaia.org>" << endl;
	exit(0);
}
